/*
   This is a corrected and highly reorganized version of Figure 3.3 from pages 93-94.

   This version adjusts the viewport and the world window so that the aspect ratio of the
   graph is preserved (and the shape of the graph is not distorted) and also so that the
   entire height of the program window is always used. This version uses a slider control
   so that when the program window is too narrow, and part of the left and/or right ends
   of the graph must be cut off, the slider can be used to pan the world window across
   the whole graph.
*/
#include <windows.h>
#include <math.h>
#include "glut.h"
#include "glui.h"

#include "plot-sinc.h"  // defines some global constants

void reshape(int, int);
void slider(float);

int windowHeight;         // this is set by the glut reshape() callback function
int windowWidth;          // this is set by the glut reshape() callback function

float sliderValue = 0.50; // this is set by the glui slider() callback function

void myDisplay(void)      // plot the sinc function, using world coordinates
{
   int viewportLeft, viewportBottom, viewportWidth, viewportHeight;
   double worldWidth, worldLeftEdge, worldRightEdge;
   const float pi = 3.14159265358979;
   GLfloat x;

   // specify the viewport (have to account for the slider in this version)
   if ( ((double)windowWidth)/(double)(windowHeight-20) >= aspectRatio )
   {
      viewportWidth = (int)((windowHeight-20)*aspectRatio);
      viewportLeft = (windowWidth-viewportWidth)/2;
   }
   else
   {
      viewportWidth = windowWidth;
      viewportLeft  = 0;
   }
   viewportHeight = windowHeight-20; // use entire height (minus the slider) of the window
   viewportBottom = 0;
   glViewport(viewportLeft, viewportBottom, viewportWidth, viewportHeight);

   // specify the world window
   if ( ((double)windowWidth)/(double)(windowHeight-20) >= aspectRatio )
   {
      worldWidth = graphWidth;
   }
   else
   {
      if (viewportHeight != 0)  // check for minimized program window
      {
         worldWidth =  (((double)viewportWidth/(double)viewportHeight)*graphHeight);
      }
      else
      {
         worldWidth = 0;
      }
   }
   printf("worldWidth = %f\n", worldWidth);

   // slide the world window left and right accross the graph
   worldLeftEdge = graphLeft + sliderValue*(graphWidth - worldWidth);

   // fix the world window in the middle of the graph
   //worldLeftEdge = graphleft + (graphWidth - worldWidth)/2.0;

   worldRightEdge = worldLeftEdge + worldWidth;

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluOrtho2D(worldLeftEdge, worldRightEdge, graphBottom, graphTop);

   glClear( GL_COLOR_BUFFER_BIT );

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

   glBegin(GL_LINE_STRIP);
      for(x = worldLeftEdge; x < worldRightEdge; x += 0.01)  // draw the plot
      {
         glVertex2f(x, sin(pi * x) / (pi * x));
      }
   glEnd();
   glutSwapBuffers();  // glui needs double buffering!
}


int main(int argc, char** argv)
{
   int window;
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);  // glui needs double buffering!
   // specify the screen window
   glutInitWindowSize(640,480);       // width, height (in pixels)
   glutInitWindowPosition(100,150);   // left, top (in pixels)
   window = glutCreateWindow("Sinc Function");
   glutDisplayFunc(myDisplay);
   glutReshapeFunc(reshape);

   // parent window, x, y, width, height, initial value, callback function
   gluiHorizontalSlider(window, 10, 0, -10, 20, 0.50, slider);

   glClearColor(1.0,1.0,1.0,0.0);
   glColor3f(0.0f,0.0f,1.0f);
   glLineWidth(1.0);

   glutMainLoop( );

   return 0;
}


/* The reshape() callback function */
void reshape(int width, int height)
{
   windowWidth  = width;
   windowHeight = height;

   printf("We got a \"reshape(%d, %d)\" event\n", width, height);

   gluiReshape(width, height);  // notice the difference here
}//reshape()


/* glui slider callback function */
void slider(float value)
{
   sliderValue = value;

   printf("We got a \"slider(%f)\" event\n", value);

   glutPostRedisplay();  // notice the difference here
}//slider()